@@ -315,6 +315,11 @@ int feenox_function_init(function_t *this) {
 
     if (this->mesh == NULL) {
       
+      if (this->vector_argument == NULL) {
+        feenox_push_error_message("function '%s' is not ready to be used, mind the location of SOLVE_PROBLEM", this->name);
+        return FEENOX_ERROR;
+      }
+      
       double **data_argument = NULL;
       feenox_check_alloc(data_argument = calloc(this->n_arguments, sizeof(double *)));
       for (int i = 0; i < this->n_arguments; i++) {
