@@ -979,7 +979,7 @@ int feenox_mesh_tag2index_alloc(mesh_t *this, size_t tag_min, size_t tag_max) {
 }
 
 
-int feenox_mesh_write_header_gmsh(FILE *file) {
+int feenox_mesh_write_header_gmsh(mesh_t *this, FILE *file) {
   fprintf(file, "$MeshFormat\n");
   fprintf(file, "2.2 0 8\n");
   fprintf(file, "$EndMeshFormat\n");
@@ -1030,8 +1030,7 @@ int feenox_mesh_write_mesh_gmsh(mesh_t *this, FILE *file, int no_physical_names)
       fprintf(file, "2 0 0");
     }
     // the nodes
-    unsigned int j = 0;
-    for (j = 0; j < this->element[i].type->nodes; j++) {
+    for (unsigned int j = 0; j < this->element[i].type->nodes; j++) {
       fprintf(file, " %ld", this->element[i].node[j]->tag);
     }
     fprintf(file, "\n");
