@@ -9,16 +9,16 @@ if [ -z "${functions_found}" ]; then
   exit 1;
 fi
 
-
-# t1 and t4 are separated from t2 & t3 because they needs gmsh
-checkgmsh
-
-checkpde mechanical
-gmsh -v 0 -2 ${dir}/nafems-t1.geo || exit $?
-answerfloat nafems-t1.fee 50 0.1
-exitifwrong $?
-
 checkpde thermal
-gmsh -v 0 -2 ${dir}/nafems-t4.geo || exit $?
-answer nafems-t4.fee 18.3
+
+answerzero nafems-t2-1d.fee
+exitifwrong $?
+
+answerzero nafems-t2-3d.fee
+exitifwrong $?
+
+answerfloat nafems-t3-1d.fee 36.6 0.05
+exitifwrong $?
+
+answerfloat nafems-t3-3d.fee 36.6 0.05
 exitifwrong $?
